package at.fhooe.mc.magicwand.client.request;

import at.fhooe.mc.magicwand.client.exceptions.CompassException;
import at.fhooe.mc.magicwand.client.exceptions.GPSException;
import at.fhooe.mc.magicwand.client.sensors.*;

/**
 * @class RequestCreator
 * @author Manuela Vogler
 * @brief Creates new request messages, combining sensor data and request options.
 */
public class RequestCreator {

    /**
     * SensorDataContainer that is used to get GPS position and compass angle
     */
    private SensorDataContainer m_container;

    /**
     * Maximum distance of POI in meters
     */
    private String m_maxDis;

    /**
     * Accuracy of search, can be high or low
     */
    private String m_disOpt;

    /**
     * Maximum number of POI to be displayed
     */
    private String m_maxPoi;

    /**
     * Default value for distance
     */
    private String m_stdValueDistance = "1000";

    /**
     * Default value for maximum number of POI
     */
    private String m_stdValueMaxPoi = "15";

    /**
     * @fn public RequestCreator() throws CompassException
     * @brief Constructs a new RequestCreator and initializes the SensorDataContainer.
     * @throws CompassException if compass connection can't be established
     */
    public RequestCreator() throws CompassException {
        try {
            m_container = new SensorDataContainer();
        } catch (CompassException _e) {
            throw new CompassException("Compass connection couldn't be established");
        }
    }

    /**
     * @fn public String createRequest (String _maxDistance, String _distanceOption, String _maxPoi) throws GPSException, CompassException
     * @brief Creates a String with the request message in XML, which contains request options and sensor data. Before creating the message, sensor data is updated.
     * @param _maxDistance Maximum distance of POI, set in options screen (null if not set)
     * @param _distanceOption Accuracy of search, set in options screen (null if not set)
     * @param _maxPoi Maximum number of POI to be displayed, set in options screen (null if not set)
     * @return The message with the request in XML
     * @throws GPSException if coordinates can't be updated or another GPS error occurs
     * @throws CompassException if an error occurs while getting the compass data
     */
    public String createRequest (String _maxDistance, String _distanceOption, String _maxPoi) throws GPSException, CompassException {
        if (_maxDistance == null) {//no option set
            m_maxDis = m_stdValueDistance;
        } else {
            m_maxDis = _maxDistance;
        }

        if (_distanceOption == null || _distanceOption.equals("precise")) {//no option set or option set to high
            m_disOpt = "high";
        } else if (_distanceOption.equals("generous")) {//option set to low
            m_disOpt = "low";
        }

        if (_maxPoi == null) {//no option set
            m_maxPoi = m_stdValueMaxPoi;
        } else {
            m_maxPoi = _maxPoi;
        }
        m_container.updateData(); //update GPS position and compass data
        return createXmlString(); //create the request message
    }

    /**
     * @fn private String createXmlString() throws CompassException, GPSException
     * @brief Creates the request message in XML format with the current sensor data and request options.
     * @return String with the request message in XML
     * @throws CompassException if compass angle is invalid
     * @throws GPSException if GPS coordinates are invalid
     */
    private String createXmlString() throws CompassException, GPSException {
        StringBuffer request = new StringBuffer();
        double lat = m_container.getLatitude();
        double lon = m_container.getLongitude();
        if (lat == 0.0 && lon == 0.0) { //no valid values available --> error must have occured
            throw new GPSException("GPS-data not available");
        }
        String angle = m_container.getAngle();
        if (angle.equals("-1")) { //no valid value available --> error must have occured
            throw new CompassException("Compass not available");
        }
        //create the message, appending XML tags
        request.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><magic-wand><poi>");
        request.append("<latitude>" + lat + "</latitude>");
        request.append("<longitude>" + lon + "</longitude>");
        request.append("<compass-data>" + angle + "</compass-data>");
        request.append("</poi><options>");
        request.append("<distance>" + m_maxDis + "</distance>");
        request.append("<accuracy>" + m_disOpt + "</accuracy>");
        request.append("<maxpoi>" + m_maxPoi + "</maxpoi>");
        request.append("</options></magic-wand>");
        return request.toString();
    }

    /**
     * @fn public void closeCompassSocket() throws CompassException
     * @brief Closes the socket to the Symbian compass application by using the SensorDataContainer.
     * @throws CompassException if connection to compass can't be closed
     */
    public void closeCompassSocket()throws CompassException{
       try {
        m_container.close();
       } catch (CompassException _e){
           throw new CompassException("Compass could not be stopped");
       }
    }

    /**
     * @fn public void set_receive(boolean _receive)
     * @brief sets receiving active or inactive
     * @param _receive set receiving active or inactive
     */
    public void set_receive(boolean _receive) {
        m_container.set_receive(_receive);
    }

}
